@@ -20,6 +20,8 @@ import pyensembl
 import varcode
 from varcode.effects import Intergenic
 
+from . import effect_ordering
+
 IUPAC = {
     "A": ["A"],
     "C": ["C"],
@@ -67,64 +69,94 @@ def reverse_base(base):
     return "".join([COMPLEMENT[b] for b in base][::-1])
 
 
-def mut2eff(chrom, pos, strand, ref, alt, genome, biotype):
+def mut2eff(chrom, pos, strand, ref, alt, genome, strandness):
 
     chrom = str(chrom).replace("chr", "")
     pos = int(pos)
     ref = ref.upper()
     alt = alt.upper()
+
     if chrom not in genome.contigs():
-        return None
-    effs = varcode.EffectCollection(
-        [
-            e
-            for b in expand_base(alt)
-            if b != ref
-            for e in varcode.Variant(
-                contig=chrom, start=pos, ref=ref, alt=b, ensembl=genome
-            ).effects()
-        ]
-    )
+        return varcode.EffectCollection([])
+
+    eff_list = [
+        e
+        for b in expand_base(alt)
+        if b != ref
+        for e in varcode.Variant(
+            contig=chrom, start=pos, ref=ref, alt=b, ensembl=genome
+        ).effects()
+    ]
 
     # filter the correct strand
-    if biotype == "RNA" and strand in "+-":
-        eff_list = []
-        for e in effs:
-            if type(e).__name__ == "Intergenic":
-                eff_list.append(e)
-            elif e.transcript is None:
-                eff_list.append(e)
-            elif e.transcript.strand == strand:
-                eff_list.append(e)
-        if len(eff_list) == 0:
-            e = Intergenic(
-                varcode.Variant(
-                    contig=chrom,
-                    start=pos,
-                    ref=ref,
-                    alt=[b for b in expand_base(alt)][0],
-                    ensembl=genome,
-                )
-            )
-            d2g = float("inf")
-            for g in genome.genes_at_locus(
-                chrom, pos - 10000, end=pos + 10000, strand=strand
+    if strandness and strand in "+-":
+        eff_list_filtered = []
+        for e in eff_list:
+            if (
+                type(e).__name__ == "Intergenic"
+                or e.transcript is None
+                or e.transcript.strand == strand
             ):
-                d = min(g.start - pos, pos - g.end)
-                if d < d2g:
-                    d2g = d
-                    e.gene = g
-                    if hasattr(g, "transcript"):
-                        e.transcript = g.transcripts[0]
-            eff_list = [e]
-        effs = varcode.EffectCollection(eff_list)
+                eff_list_filtered.append(e)
+    else:
+        eff_list_filtered = eff_list
+
+    # set as intergenic if no transcript is found
+    if len(eff_list_filtered) == 0:
+        e = Intergenic(
+            varcode.Variant(
+                contig=chrom,
+                start=pos,
+                ref=ref,
+                alt=[b for b in expand_base(alt)][0],
+                ensembl=genome,
+            )
+        )
+        d2g = float("inf")
+        expand_intergenic_window = 10000
+        for g in genome.genes_at_locus(
+            chrom,
+            pos - expand_intergenic_window,
+            end=pos + expand_intergenic_window,
+            strand=strand,
+        ):
+            d = min(g.start - pos, pos - g.end)
+            if d < d2g:
+                d2g = d
+                e.gene = g
+                if hasattr(g, "transcript"):
+                    e.transcript = g.transcripts[0]
+        eff_list_filtered = [e]
+
+    effs = varcode.EffectCollection(eff_list_filtered)
 
     return effs
 
 
+# parse these effects in `parse_eff()` fuction
+REPORT_FEATURES = [
+    "mut_type",
+    "gene_type" "gene_name",
+    "gene_pos",
+    "transcript_name",
+    "transcript_pos",
+    "transcript_motif",
+    "coding_pos",
+    "codon_ref",
+    "aa_pos",
+    "aa_ref",
+    "distance2splice",
+]
+
+
 def parse_eff(eff, pos, pad):
+    number_of_features = len(REPORT_FEATURES)
+    if eff is None:
+        return ["NotInReferenceGenome", *([None] * (number_of_features - 1))]
     pos = int(pos)
     mut_type = type(eff).__name__
+    # calculate the distance to the splice site
+    # TODO: check exon number
     d2s = []
     if eff.transcript is None:
         distance2splice = None
@@ -150,10 +182,44 @@ def parse_eff(eff, pos, pad):
         # minus distance to splice site
         distance2splice = sorted(d2s, key=lambda x: abs(x))[0]
 
-    transcript_id = None if mut_type == "Intergenic" else eff.transcript_id
+    try:
+        gene_type = eff.gene.biotype
+    except:
+        gene_type = None
+    # put gene name and gene id together
+    gene_name = getattr(eff, "gene_id", None)
+    if gene_name is not None and hasattr(eff, "gene_name") and eff.gene_name:
+        gene_name = gene_name + "(" + eff.gene_name + ")"
 
-    non_exon_recored = [mut_type, eff.gene_name, transcript_id] + [None] * 7
-    if mut_type in ["Intergenic", "Intronic", "SpliceDonor"]:
+    # put transcript name and transcript id together
+    transcript_name = getattr(eff, "transcript_id", None)
+    if (
+        transcript_name is not None
+        and hasattr(eff, "transcript_name")
+        and eff.transcript_name
+    ):
+        transcript_name = transcript_name + "(" + eff.transcript_name + ")"
+
+    # NOTE: report distaance to gene and transcript
+    # But how to store the nubmer? Negetive for upstream? But what for downstream?
+    if mut_type == "Intergenic":
+        return [mut_type, gene_type, gene_name, None, transcript_name] + [
+            None
+        ] * (number_of_features - 4)
+
+    try:
+        gene_pos = eff.gene.offset(pos) + 1
+    except:
+        gene_pos = None
+
+    non_exon_recored = [
+        mut_type,
+        gene_type,
+        gene_name,
+        gene_pos,
+        transcript_name,
+    ] + [None] * (number_of_features - 4)
+    if mut_type in ["Intronic", "SpliceDonor"]:
         return non_exon_recored
     try:
         transcript_pos = eff.transcript.spliced_offset(pos) + 1
@@ -207,8 +273,10 @@ def parse_eff(eff, pos, pad):
 
     return [
         mut_type,
-        eff.gene_name,
-        transcript_id,
+        gene_type,
+        gene_name,
+        gene_pos,
+        transcript_name,
         transcript_pos,
         transcript_motif,
         coding_pos,
@@ -220,13 +288,23 @@ def parse_eff(eff, pos, pad):
 
 
 def site2mut(
-    chrom, pos, strand, ref, alt, genome, biotype, pad=10, all_effects=False
+    chrom,
+    pos,
+    strand,
+    ref,
+    alt,
+    genome,
+    pad=10,
+    strandness=True,
+    all_effects=False,
+    pU_mode=False,
 ):
-    effs = mut2eff(chrom, pos, strand, ref, alt, genome, biotype)
-    if effs is None:
-        return [[None] * 10]
+    effs = mut2eff(chrom, pos, strand, ref, alt, genome, strandness)
+
+    if len(effs) == 0:
+        return [parse_eff(None, pos, pad)]
     if not all_effects:
-        eff = effs.top_priority_effect()
+        eff = effect_ordering.get_top_effect(effs, pU_mode=pU_mode)
         return [parse_eff(eff, pos, pad)]
     return [parse_eff(eff, pos, pad) for eff in effs]
 
@@ -262,12 +340,24 @@ def site2mut(
     help="ensembl release",
     required=False,
 )
+# for backward compartable
 @click.option(
     "--type",
     "-t",
-    "biotype",
+    "dna_or_rna",
     type=click.Choice(["DNA", "RNA"], case_sensitive=False),
     default="DNA",
+    help="(deprecated)",
+)
+@click.option(
+    "--strandness", "-s", help="Use strand infomation or not?", is_flag=True
+)
+@click.option(
+    "--pU-mode",
+    "-u",
+    "pU_mode",
+    help="Make rRNA, tRNA, snoRNA into top priority.",
+    is_flag=True,
 )
 @click.option(
     "--npad",
@@ -296,12 +386,17 @@ def run(
     output,
     reference,
     release,
-    biotype,
     npad,
+    strandness,
+    dna_or_rna,
     all_effects,
+    pU_mode,
     with_header,
     columns,
 ):
+    if dna_or_rna == "RNA":
+        strandness = True
+
     ensembl_genome = pyensembl.EnsemblRelease(
         release=release, species=reference
     )
@@ -312,18 +407,7 @@ def run(
         ensembl_genome.index()
 
     with open(input, "r") as f:
-        annot_header = [
-            "mut_type",
-            "gene_name",
-            "transcript_id",
-            "transcript_pos",
-            "transcript_motif",
-            "coding_pos",
-            "codon_ref",
-            "aa_pos",
-            "aa_ref",
-            "distance2splice",
-        ]
+        annot_header = REPORT_FEATURES
         if with_header:
             input_header = f.readline().strip().split()
         else:
@@ -341,7 +425,7 @@ def run(
             s = input_cols[columns[2] - 1]
             ref = input_cols[columns[3] - 1]
             alt = input_cols[columns[4] - 1]
-            if biotype == "RNA":
+            if strandness:
                 if s == "-":
                     ref = reverse_base(ref.upper())
                     alt = reverse_base(alt.upper())
@@ -352,7 +436,16 @@ def run(
                     raise ValueError("Strand must be + or -")
 
             annot_list = site2mut(
-                c, p, s, ref, alt, ensembl_genome, biotype, npad, all_effects
+                c,
+                p,
+                s,
+                ref,
+                alt,
+                ensembl_genome,
+                npad,
+                strandness,
+                all_effects,
+                pU_mode,
             )
             for annot in annot_list:
                 annot_str = list(map(str, annot))
